##########################################################################
# Copyright (c) 2012 W. Evan Johnson, Boston University School of Medicine
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##########################################################################

import glob,os,sys,time,posix
import utilities
from mycel import MyCEL
from normalize import *

def getMetadata(probeSequenceTabFilePath):
    coord2probe = {}
    probe2seq = {}
    probe2type = {}

    probeSequenceTabFile = open(probeSequenceTabFilePath)

    for line in probeSequenceTabFile:
        lineItems = line.rstrip().split("\t")
        probe_id = lineItems[0]
        probe_coor_x = int(lineItems[1])
        probe_coor_y = int(lineItems[2])
        probe_seq = lineItems[3]
        probe_type = lineItems[4]

        coord2probe[(probe_coor_x, probe_coor_y)] = probe_id
        probe2seq[probe_id] = probe_seq
        probe2type[probe_id] = probe_type

    probeSequenceTabFile.close()

    return coord2probe, probe2seq, probe2type

celFilePath = sys.argv[1]
celFileName = os.path.basename(celFilePath)
probeSequenceTabFilePath = sys.argv[2]
outFilePath = sys.argv[3]

outputRawIntensities = False
if len(sys.argv) >= 5:
    outputRawIntensities = sys.argv[4] == "True"

cel = MyCEL()
norm = Normalize()

print "Reading annotations from %s " % probeSequenceTabFilePath
coord2probe, probe2seq, probe2type = getMetadata(probeSequenceTabFilePath)

print "Reading " + celFilePath
probe2intensity = cel.read_cel(celFilePath, coord2probe)

print "Normalizing to " + outFilePath
normValues = norm.normalize(probe2intensity, probe2seq, probe2type, outFilePath, outputRawIntensities)
